/* Dialog / Popup ===================================== */

// dialog names (used as dialog identifiers)
const dialogNames = {
    "promptToShare" : "promptToShareData",
    "resetData" : "resetData",
    "blockSites" : "blockSites",
    "hideSites" : "hideSites",
    "startUploadData" : "startUploadData",
    "stopUploadData" : "stopUploadData",
    "privateBrowsing" : "privateBrowsing",
    "saveOldData": "saveOldData"
};

const allDialogs = {
    'Reset Data Confirmation': confirmResetDataDialog,
    'Block Sites Confirmation': confirmBlockSitesDialog,
    'Hide Sites Confirmation': confirmHideSitesDialog,
    'Upload Data Confirmation': askForDataSharingConfirmationDialog,
    'Stop Uploading Data Confirmation': stopSharingDialog,
    'Private Browsing Notification': informUserOfUnsafeWindowsDialog,
    'Save Data From Earlier Format': promptToSaveOldDataDialog,
    'Help the Ecosystem by Sharing': showPromptToShareDialog
};





// options: name, title, message, type, dnsPrompt(Do Not Show), imageUrl
function dialog(options,callback){
    if ( doNotShowDialog(options.name) ) return; // according to user pref, do not show this dialog
    createDialog(options,callback);
}

function doNotShowDialog(dialogName){
    var dnsPref = localStorage.dnsDialogs || "[]";
    dnsPref = JSON.parse(dnsPref);
    return ( dnsPref.indexOf(dialogName) > -1 ) ? true : false;
}

function createDialog(options,callback){
    var modal = picoModal({
        content: createDialogContent(options),
        closeButton: false,
        overlayClose: false,
        overlayStyles: {
            backgroundColor: "#000",
            opacity: 0.75
        }
    });

    addDialogEventHandlers(modal,options,function(userResponse){
        callback(userResponse);
    });
}

function createDialogContent(options){
    return  dialogTitleBar(options) +
            dialogMessage(options) +
            dialogControls(options);
}

function dialogTitleBar(options){
    return "<div class='dialog-title'>" + (options.title || "&nbsp;") + "</div>";
}

function dialogMessage(options){
    return  "<div class='dialog-content'>" + 
                (options.imageUrl ? "<div class='dialog-sign'><img src='" + options.imageUrl + "' /></div>" : "") + 
                "<div class='dialog-message'>" + (options.message || "&nbsp;") + "</div>" + 
            "</div>";
}

function dialogControls(options){
    var doNotShowAgainPrompt = "<div class='dialog-dns'><input type='checkbox' /> Nicht mehr zeigen.</div>";
    // control buttons
    var controlButtons = "<div class='dialog-btns'>";
    var okButton = "<a class='pico-close dialog-ok'>OK</a>";
    var cancelButton = "<a class='pico-close dialog-cancel'>Abbrechen</a>";
    // check dialog type
    // alert dialog only needs a single button - "OK"
    // else we show both "OK" and "Cancel" buttons
    if ( options.type == "alert" ){
        controlButtons += "<a class='pico-close dialog-ok'>OK</a>";

    }else{
        if ( navigator.appVersion.indexOf("Win") > -1 ){ // runs on Windows
            controlButtons += okButton + cancelButton;
        }else{ // runs on OS other than Windows
            controlButtons += cancelButton + okButton;
        }
    }
    controlButtons += "</div>";

    return  "<div class='dialog-controls'>" + 
                ( options.dnsPrompt ? doNotShowAgainPrompt : "" ) +
                controlButtons + 
            "</div>";
}

function addDialogEventHandlers(modal,options,callback){
    var dialogContainer = modal.modalElem;
    // OK button click event handler
    var okButton = dialogContainer.querySelector(".pico-close.dialog-ok");
    okButton.addEventListener("click",function(){
        if ( dialogContainer.querySelector(".dialog-dns input") && dialogContainer.querySelector(".dialog-dns input").checked ){ // Do Not Show
            addToDoNotShowAgainList(options.name);
        }
        modal.close();
        callback(true);
    });
    // Cancel button click event handler
    var cancelButton = dialogContainer.querySelector(".pico-close.dialog-cancel");
    if ( cancelButton ){
        cancelButton.addEventListener("click",function(){
            if (options.name == dialogNames.promptToShare){
                if ( dialogContainer.querySelector(".dialog-dns input").checked ){
                    addToDoNotShowAgainList(options.name);
                }
            }
            modal.close();
            callback(false);
        });
    }

    var keyDownHandler = function(event){
        // disable Tab
        if ( event.keyCode == "9" ){ 
            event.preventDefault();
        }
        // press Esc to close the dialog (functions the same as clicking Cancel)
        if ( event.keyCode == "27" ){ // Esc key pressed
            modal.close();
            callback(false);
        }
    }
    document.addEventListener("keydown", keyDownHandler);

    modal.onClose(function(){
        document.removeEventListener("keydown", keyDownHandler);
    });

    // for Upload Data dialog
    if (dialogContainer.querySelector(".toggle-pp")){
        dialogContainer.querySelector(".toggle-pp").addEventListener("click",function(event){
            dialogContainer.querySelector(".pico-content .privacy-policy").classList.toggle("collapsed");
        });
    }

    restrictTabWithinDialog(modal);
}


function addToDoNotShowAgainList(dialogName){
    var dnsPref = localStorage.dnsDialogs || "[]";
    if (dnsPref === 'undefined'){
        dnsPref = "[]";
    }
    dnsPref = JSON.parse(dnsPref);
    dnsPref.push(dialogName);
    localStorage.dnsDialogs = JSON.stringify(dnsPref);
}


function restrictTabWithinDialog(modal){
    var dialogContainer = modal.modalElem;
    assignTabIndices(modal);
    dialogContainer.addEventListener("keypress", function(event){
        event.stopPropagation();
        var focusedElm = document.activeElement;
        // Tab key is pressed
        if ( event.keyCode == "9" ){
            var currentTabIndex = parseInt(focusedElm.getAttribute("tabIndex"));
            var nextElem = dialogContainer.querySelector("[tabIndex='" + (currentTabIndex+1) + "']");
            if ( nextElem ){
                nextElem.focus();
            }else{
                dialogContainer.querySelector("[tabIndex='0']").focus();
            }
        }
        // when the focused element is the OK or Cancel button and Enter key is pressed
        // mimic mouse clicking on button
        if ( event.keyCode == "13" && focusedElm.mozMatchesSelector(".pico-content .dialog-btns a") ){
            focusedElm.click();
        }
    });
}

function assignTabIndices(modal){
    var dialogContainer = modal.modalElem;
    var allElemsInDialog = dialogContainer.querySelectorAll("*");
    var tabIndex = 0;
    toArray(allElemsInDialog).forEach(function(elem, i){
        if ( elem.nodeName.toLowerCase() == "a" || elem.nodeName.toLowerCase() == "input" ){
            elem.setAttribute("tabIndex", tabIndex);
            tabIndex++;
        }
    });
    dialogContainer.querySelector("[tabIndex='0']").focus();
}

function askForDataSharingConfirmationDialog(callback){
    dialog( {   "name": dialogNames.startUploadData,
            "title": "Daten hochladen", 
            "message": 
                '<p>Sie sind dabei, mit dem Hochladen on Daten auf den Lightbeam-Server zu beginnen. Ihre Daten werden weiter periodisch hochgeladen, bis das Hochladen abgeschaltet wird. Für weitere Informationen über die Daten, die wir hochladen, welche Schritte wir unternehmen, um das Risiko zu minimieren, dass Sie identifiziert werden und welche Datenschutzrichtlinien Mozilla hat, lesen Sie bitte in den <a class="toggle-pp">Lightbeam-Datenschutzrichtlinien</a>.</p>' + 
                // Lightbeam Privacy Policy.
                '<div class="privacy-policy"> <header><b>Lightbeam Datenschutz-Vermerk</b></header> <p> Wir sorgen uns um Ihre Privatsphäre. Lightbeam ist eine Browser-Erweiterung, die Anfragen Dritter auf einer Site sammelt, die Sie besuchen und sie visualisieren hilft. Wenn Sie Lightbeam-Daten an Mozilla (also an uns) senden möchten, beschreiben unsere <a href="#mozillaprivacypolicy">Datenschutzrichtlinien</a>, wie wir mit den Daten umgehen. </p> <header><b>Was Sie wissen sollten</b></header> <ul class="bullet-form"> <li> Nach der Installation von Lightbeam sammelt die Erweiterung Daten, um Ihnen zu helfen, Anfragen von Drittanbietern zu visualisieren, wenn Sie Sites besuchen. <ul> <li>Wenn Sie eine Site besuchen, und die Site mit einem Drittanbieter Kontakt aufnimmt, sammelt Lightbeam den folgenden Datentyp: Domains der besuchten Sites und der Drittanbieter, die Existenz von Cookies und einen groben Zeitstempel davon, wann die Site besucht wurde. Um eine vollständige Liste zu sehen, besuchen Sie bitte <a href="https://github.com/mozilla/lightbeam/blob/master/doc/data_format.v1.1.md">diese Seite.</a></li> </ul> </li> <li> Standardmäßig verbleiben von Lightbeam gesammelte Daten in Ihrem Browser und werden nicht an uns gesandt. </li> <li> Sie können Ihre Lightbeam-Daten mit uns teilen. Daten aus Lightbeam können uns und anderen helfen, die Beziehungen von Drittanbietern im Web zu verstehen und die weitere Forschung auf dem Gebiet des Online-Trackings und Datenschutzes voranzutreiben. <ul> <li>Wenn Sie sich entschließen dazu beizutragen, wird Ihr Browser uns Lightbeam-Daten senden (eine Liste der einbezogenen Datenart können Sie <a href="https://github.com/mozilla/lightbeam/blob/master/doc/data_format.v1.1.md">hier sehen.</a>). Wir werden Ihre Daten zusammen mit den Daten von anderen so an eine umfassende und offene Datenbank senden, dass wir glauben, das Risiko einer Wiedererkennung zu minimieren. Die offene Verfügbarkeit dieser Daten kan Nutzern und Forschern helfen, sachkundigere Entscheidungen auf der Grundlage der gesammelten Informationen zu treffen. </li> <li>Die Deinstallation von Lightbeam verhindert das Sammeln von weiteren Lightbeam-Daten.</li><li> Klicken Sie auf "Daten zurücksetzen", um die lokal in Ihrem Browser gespeicherten Daten zu löschen. </li> </ul> </li> </ul> <header><b>Mozilla-Datenschutzrichtlinien &ndash; Weitere Informationen</b></header> <p>Ihre Privatsphäre ist ein wichtiger Faktor, den Mozilla (das sind wir) bei der Entwicklung von jedem unserer Produkte berücksichtigt. Wir sind verpflichtet, transparent und offen zu sein und möchten, dass Sie wissen, wie wir Informationen über Sie erhalten und was wir mit diesen Informationen tun, sobald wir sie haben.</p> <header><b>Was meinen wir mit "persönlichen Informationen"?</b></header> <p> Für uns sind "persönliche Informationen" Informationen, die Sie identifizieren, wie zum Beispiel Ihr Name oder Ihre E-Mail-Adresse. </p> <p> Informationen, die nicht darunter fallen, sind "nichtpersönliche Informationen". </p> <p> Wenn wir Ihre persönlichen Informationen zusammen mit nichtpersönlichen Informationen speichern, werden wir diese Kombination als persönliche Information betrachten. Wenn wir alle persönlichen Informationen aus einem Datensatz entfernen, dann sind die verbleibenden Informationen nichtpersönliche Informationen. </p> <header><b>Wie erfahren wir Informationen über Sie?</b></header> <p> Wir erfahren Informationen über Sie, wenn: </p> <ul> <li> Sie sie uns direkt geben (z.B. wenn Sie uns Absturzmeldungen aus Firefox senden); wir sammeln sie automatisch über unsere Produkte und Dienste  (z.B. wenn wir prüfen, ob Ihre Firefox-Version aktuell ist); </li> <li> wir sammeln sie automatisch über unsere Produkte und Dienste  (z.B. wenn wir prüfen, ob Ihre Firefox-Version aktuell ist); </li> <li> uns irgendjemand Informationen über Sie mitteilt (z.B., Thunderbird arbeitet mit Ihren E-Mail-Anbietern zusammen, um Ihr Konto einzurichten) oder </li> <li> wenn wir versuchen, auf der Grundlage der Informationen, die Sie uns gegeben haben, mehr von Ihnen zu verstehen (z.B. wenn wir Ihre IP-Adresse benutzen, um die Sprache für unsere Dienste anzupassen). </li> </ul> <br/><header><b>Was machen wir mit Ihren Informationen, sobald wir sie haben? </b></header> <p>Wenn Sie uns persönliche Informationen geben, werden wir sie für die Zwecke verwenden, für die Sie uns die Erlaubnis gegeben haben. Im Allgemeinen verwenden wir Ihre Informationen, damit sie uns helfen, unsere Produkte und Dienste zur Verfügung zu stellen und für Sie zu verbessern.</p> <header><b>Wann teilen wir Ihre Informationen mit anderen?</b></header> <ul> <li>Wenn wir Ihre Erlaubnis haben, sie mit ihnen zu teilen.</li> <li>Um Produkte und Dienste für Sie zu verarbeiten oder bereitzustellen, aber nur, wenn die betreffenden Dritten sich vertraglich dazu verpflichtet haben, die Daten so zu behandeln, wie es Mozilla genehmigt hat.</li> <li>Wenn wir unsere <a href="https://www.mozilla.org/en-US/about/manifesto/">Mission erfüllen, offen zu sein</a>. Wir geben manchmal Informationen frei, um unsere Produkte zu verbessern und ein offenes Web zu fördern, aber wenn wir das tun, werden wir Ihre persönlichen Informationen entfernen und sie so offenbaren, dass das Risiko einer Identifizierung minimiert wird.</li> <li>Wenn es gesetzlich gefordert wird. Wir befolgen das Gesetz, wann auch immer wir Anfragen zu Ihnen von der Regierung oder in Zusammenhang mit einem Gerichtsprozess erhalten. Wir werden Sie benachrichtigen, wenn wir gebeten werden, ihre persönlichen Informationen diesbezüglich herauszugeben, es sei denn, uns wird gesetzlich untersagt, dies zu tun. Wenn wir Anfragen wie diese erhalten, werden wir nur persönliche Informationen freigeben, wenn wir guten Glaubens sind, dass das Gesetz von uns verlangt, dies zu tun. Nichts in diesen Richtlinien beabsichtigt gesetzliche Einwände zu begrenzen, die Sie möglicherweise gegen eine Anfrage von einem Drittanbieter haben, der Ihre Informationen offenbaren möchte.</li> <li>Wenn wir glauben, dass es notwendig ist, Schaden von Ihnen oder jemand anderem fernzuhalten. Wir werden unsere Informationen diesbezüglich nur teilen, wenn wir guten Glaubens sind, dass es vernünftigerweise notwendig ist, die Rechte, das Eigentum oder die Sicherheit von Ihnen, anderen Nutzern, Mozillas oder der Öffentlichkeit zu schützen.</li> <li>Wenn sich unsere Organisationsstruktur oder unser Status ändert (wenn wir einer Umstrukturierung unterliegen, von einem anderen Unternehmen erworben werden oder Konkurs gehen) müssen wir möglicherweise Ihre Informationen an den Nachfolger oder Miteigentümer übergeben.</li><br/> </ul> <header><b>Wie speichern und schützen wir Ihre persönlichen Informationen? </b></header> <p> Wir sind verpflichtet, Ihre persönlichen Informationen zu schützen, sobald wir sie haben. Wir ergreifen dazu physische, geschäftliche und technische Sicherheitsmaßnahmen. Ungeachtet unserer Anstrengungen: Wenn wir von einem Sicherheitsleck erfahren, werden wir Sie benachrichtigen, damit Sie geeignete Schutzvorkehrungen treffen können.</p> <p>Wir wollen auch nicht Ihre persönlichen Informationen länger behalten, als wir sie brauchen, deshalb werden wir sie nur so lange behalten, um das zu tun, wofür sie gesammelt wurden. Sobald wir sie nicht mehr benötigen, werden wir Schritte unternehmen, sie zu vernichten, es sei denn, wir werden gesetzlich gezwungen, sie länger aufzubewahren.</p> <header><b>Was sollten Sie sonst noch wissen? </b></header> <p>Wir sind eine globale Organisation und unsere Computer stehen an mehreren unterschiedlichen Plätzen auf der ganzen Welt. Wir nutzen auch Diensteanbieter, deren Computer sich ebenfalls in verschiedenen Ländern befinden können. Das bedeutet, dass Ihre Informationen auf einen dieser Computer in einem anderen Land gelangen können, und dieses Land hat möglicherweise ein anderes Niveau zur Regelung des Datenschutzes. Indem Sie uns Ihre Informationen geben, willigen Sie in diese Art der Weitergabe Ihrer Informationen ein. Egal in welchem Land sich Ihre Informationen befinden, wir halten uns an das geltende Recht und werden uns ebenfalls an die Verpflichtungen halten, die wir in diesen Datenschutzrichtlinien eingehen.<p> <p>Wenn Sie jünger als 13 Jahre sind, wollen wir Ihre persönlichen Informationen nicht, und Sie müssen sie uns nicht überlassen. Wenn Sie ein Elternteil sind und glauben, dass Ihr Kind, das jünger als 13 Jahre ist, uns mit persönlichen Informationen versorgt hat, setzen Sie sich bitte mit uns über <a href="mailto:lightbeam-privacy@mozilla.org">lightbeam-privacy@mozilla.org</a> in Verbindung, damit die Informationen Ihres Kindes gelöscht werden.</p> <header><b>Was ist, wenn wir diese Richtlinien ändern? </b></header> <p> Es ist möglich, dass wir diese Richtlinien ändern müssen. Falls wir das tun, werden wir Sie benachrichtigen. </p> </div>' + 
                // Lightbeam Privacy Policy ends
                '<br />' +
                '<p>Mit dem Klick auf OK erklären Sie sich mit den Verfahrensweisen zu den Daten in unserem Datenschutz-Vermerk einverstanden.</p>',
            "imageUrl": "image/lightbeam_popup_warningsharing.png"
    },
    callback);
}

function stopSharingDialog(callback){
    dialog( {   "name": dialogNames.stopUploadData,
                "title": "Hochladen der Daten beenden", 
                "message": 
                    '<p>Sie beenden hiermit das Hochladen von Daten auf den Lightbeam-Server.</p>' +
                    '<p>Nach dem Klick auf OK werden keine Daten mehr hochgeladen.</p>',
                "imageUrl": "image/lightbeam_popup_stopsharing2.png"
            },
            function(confirmed){
                if ( confirmed ){
                    localStorage.userHasOptedIntoSharing = false;
                    if (uploadTimer){
                        clearTimeout(uploadTimer);
                        uploadTimer = null;
                    }
                }
                callback(confirmed);
            }
    );
}

function informUserOfUnsafeWindowsDialog(){
    dialog( {
        "type": "alert",
        "name": dialogNames.privateBrowsing,
        "dnsPrompt": true,
        "title": "Privatmodus",
        "message": "<p>Sie haben ein oder mehrer Fenster im privaten Modus geöffnet.</p>" +
                    "<p>Verbindungen im privaten Modus werden in Lightbeam visualisiert, aber die Daten werden weder lokal gespeichert noch jemals geteilt, selbst wenn Teilen aktiviert ist.</p>" +
                    "<p> Informationen, die im privaten Modus gesammelt werden, werden gelöscht, wenn Lightbeam neu gestartet wird, und werden überhaupt nicht gesammelt, wenn Lightbeam nicht geöffnet ist.</p>",
        "imageUrl": "image/lightbeam_popup_privacy.png"
        },
        function(confirmed){}
    );
}


/******************************************
*  Prompt to save data from older Collusion format
*/

function promptToSaveOldDataDialog(data){
    dialog({
        "type": "message",
        "name": dialogNames.saveOldData,
        "dnsPrompt": false,
        "title": "Daten in einem alten Format speichern",
        "message": "<p>Lightbeam wurde in einem neuen Datenformat aktualisiert.</p>" + 
                   "<p>Die alten Daten, die Sie mit der Beta (Collusion) gespeichert haben, werden nicht mehr unterstützt und werden gelöscht.</p>" + 
                   "<p>Wenn Sie eine Kopie der alten Daten speichern möchten, bevor sie gelöscht werden, klicken Sie auf OK. Wenn Sie auf Abbrechen klicken, werden die alten Daten gelöscht.</p>"
    },
    function(confirmed){
        if (confirmed){
            downloadAsJson(data, 'oldformatCollusionData.json');
        }
    });

}

function confirmBlockSitesDialog(callback){
    dialog( {   "name" : dialogNames.blockSites,
                "title": "Sites blocken",
                "message":  "<p><b>Warnung:</b></p> " +
                            "<p>Das Blocken von Sites verhindert, dass jedweder Inhalt von ausgewählten Domains, zum Beispiel: [example.com, example.net] und alle ihre Subdomains [mail.example.com, news.example.net usw.] geladen wird. </p>" +
                            "<p>Dadurch können bestimmte Sites nicht mehr funktionieren und Ihre Arbeit mit dem Internet einschränken. Verwenden Sie daher diese Funktion mit Vorsicht. </p>",
                "imageUrl": "image/lightbeam_popup_blocked.png"
            },
            callback
    );
}

function confirmHideSitesDialog(callback){
    dialog( {   
        "name": dialogNames.hideSites,
        "dnsPrompt": true,
        "title": "Sites verbergen",
        "message":  "<p>Diese Site werden nicht in den Lightbeam-Visualisierungen gezeigt, einschließlich der Listen-Ansicht, es sei denn, Sie schalten diese Funktion wieder um, indem Sie auf die Schaltfläche Versteckte Sites zeigen klicken.</p>" +
                    "<p>Sie können diese nutzen, um vertrauenswürdige Sites aus den Daten auszuschließen.</p>",
        "imageUrl": "image/lightbeam_popup_hidden.png"
        },
        callback
    );
}

function confirmResetDataDialog(callback){
    dialog( {
        "name": dialogNames.resetData,
        "title": "Daten zurücksetzen",
        "message":  "<p>Mit dem Klick auf OK werden alle Lightbeam-Informationen, einschließlich der Verbindungschronik, der Benutzereinstellungen, der Blockierliste usw. gelöscht.</p>" + 
                    "<p>Ihr Browser kehrt zum Stand vor der Neuinstallation von Lightbeam zurück.</p>",
        "imageUrl": "image/lightbeam_popup_warningreset.png"
    },callback
    );
}

function showPromptToShareDialog(callback){
    dialog( {
        "name": dialogNames.promptToShare,
        "dnsPrompt": true,
        "title": "Dem Ökosystem helfen",
        "message":  "<p>Als Nutzer von Lightbeam können Sie dazu beitragen, unser Datenökosystem aufzubauen.</p>" + 
                    "<p>Indem Sie Ihre Daten mit uns teilen, können Sie uns und anderen helfen, die Beziehungen von Drittanbietern im Web zu verstehen und die weitere Forschung auf dem Gebiet des Online-Tracking und Datenschutzes voranzutreiben.</p>  "+
                    "<p>Möchten Sie Ihre Daten jetzt in die <a href='http://mozilla.org/en-US/lightbeam/database/'>öffentliche Datenbank</a> hochladen?</p>",
        "imageUrl": "image/lightbeam_popup_startsharing.png"
    },
    callback
    );

}

// Helper function for testing so we can trigger any dialog.
function showDialog(name){
    if (Object.keys(allDialogs).indexOf(name) > -1){
        var tempPref = localStorage.dnsDialogs;
        localStorage.dnsDialogs = '[]';
        allDialogs[name](function(){});
        localStorage.dnsDialogs = tempPref;
    }else{
        console.log('Use: showDialog("name") where name is one of');
        Object.keys(allDialogs).forEach(function(name){
            console.log('\t%s', name);
        });
    }
}

